@@ -84,10 +84,10 @@ static GLenum get_texture_view_target(const struct wined3d_gl_info *gl_info,
 static const struct wined3d_format *validate_resource_view(const struct wined3d_view_desc *desc,
         struct wined3d_resource *resource, BOOL mip_slice, BOOL allow_srgb_toggle)
 {
-    const struct wined3d_gl_info *gl_info = &resource->device->adapter->gl_info;
+    const struct wined3d_adapter *adapter = resource->device->adapter;
     const struct wined3d_format *format;
 
-    format = wined3d_get_format(gl_info, desc->format_id, resource->usage);
+    format = wined3d_get_format(adapter, desc->format_id, resource->usage);
     if (resource->type == WINED3D_RTYPE_BUFFER && (desc->flags & WINED3D_VIEW_BUFFER_RAW))
     {
         if (format->id != WINED3DFMT_R32_TYPELESS)
@@ -96,7 +96,7 @@ static const struct wined3d_format *validate_resource_view(const struct wined3d_
             return NULL;
         }
 
-        format = wined3d_get_format(gl_info, WINED3DFMT_R32_UINT, resource->usage);
+        format = wined3d_get_format(adapter, WINED3DFMT_R32_UINT, resource->usage);
     }
 
     if (wined3d_format_is_typeless(format))
@@ -118,7 +118,7 @@ static const struct wined3d_format *validate_resource_view(const struct wined3d_
                 return NULL;
             }
 
-            format = wined3d_get_format(gl_info, WINED3DFMT_R32_UINT, resource->usage);
+            format = wined3d_get_format(adapter, WINED3DFMT_R32_UINT, resource->usage);
             element_size = buffer->desc.structure_byte_stride;
         }
         else
@@ -169,8 +169,8 @@ static void create_texture_view(struct wined3d_gl_view *view, GLenum view_target
         const struct wined3d_view_desc *desc, struct wined3d_texture *texture,
         const struct wined3d_format *view_format)
 {
+    unsigned int level_idx, layer_idx, layer_count;
     const struct wined3d_gl_info *gl_info;
-    unsigned int layer_idx, layer_count;
     struct wined3d_context *context;
     GLuint texture_name;
 
@@ -189,20 +189,21 @@ static void create_texture_view(struct wined3d_gl_view *view, GLenum view_target
     wined3d_texture_prepare_texture(texture, context, FALSE);
     texture_name = wined3d_texture_get_texture_name(texture, context, FALSE);
 
+    level_idx = desc->u.texture.level_idx;
     layer_idx = desc->u.texture.layer_idx;
     layer_count = desc->u.texture.layer_count;
-    if (view_target == GL_TEXTURE_3D && (layer_idx || layer_count != 1))
+    if (view_target == GL_TEXTURE_3D)
     {
-        FIXME("Depth slice (%u-%u) not supported.\n", layer_idx, layer_count);
+        if (layer_idx || layer_count != wined3d_texture_get_level_depth(texture, level_idx))
+            FIXME("Depth slice (%u-%u) not supported.\n", layer_idx, layer_count);
         layer_idx = 0;
         layer_count = 1;
     }
 
     gl_info->gl_ops.gl.p_glGenTextures(1, &view->name);
     GL_EXTCALL(glTextureView(view->name, view->target, texture_name, view_format->glInternal,
-            desc->u.texture.level_idx, desc->u.texture.level_count,
-            layer_idx, layer_count));
-    checkGLcall("Create texture view");
+            level_idx, desc->u.texture.level_count, layer_idx, layer_count));
+    checkGLcall("create texture view");
 
     if (is_stencil_view_format(view_format))
     {
@@ -218,7 +219,7 @@ static void create_texture_view(struct wined3d_gl_view *view, GLenum view_target
         context_bind_texture(context, view->target, view->name);
         gl_info->gl_ops.gl.p_glTexParameteriv(view->target, GL_TEXTURE_SWIZZLE_RGBA, swizzle);
         gl_info->gl_ops.gl.p_glTexParameteri(view->target, GL_DEPTH_STENCIL_TEXTURE_MODE, GL_STENCIL_INDEX);
-        checkGLcall("Initialize stencil view");
+        checkGLcall("initialize stencil view");
 
         context_invalidate_compute_state(context, STATE_COMPUTE_SHADER_RESOURCE_BINDING);
         context_invalidate_state(context, STATE_GRAPHICS_SHADER_RESOURCE_BINDING);
