@@ -42,15 +42,6 @@
 #include "security.h"
 #include "process.h"
 
-enum pipe_state
-{
-    ps_idle_server,
-    ps_wait_open,
-    ps_connected_server,
-    ps_wait_disconnect,
-    ps_wait_connect
-};
-
 struct named_pipe;
 
 struct pipe_message
@@ -66,6 +57,8 @@ struct pipe_end
     struct object        obj;        /* object header */
     struct fd           *fd;         /* pipe file descriptor */
     unsigned int         flags;      /* pipe flags */
+    unsigned int         state;      /* pipe state */
+    struct named_pipe   *pipe;
     struct pipe_end     *connection; /* the other end of the pipe */
     process_id_t         client_pid; /* process that created the client */
     process_id_t         server_pid; /* process that created the server */
@@ -79,16 +72,13 @@ struct pipe_server
 {
     struct pipe_end      pipe_end;   /* common header for pipe_client and pipe_server */
     struct list          entry;      /* entry in named pipe servers list */
-    enum pipe_state      state;      /* server state */
-    struct pipe_client  *client;     /* client that this server is connected to */
-    struct named_pipe   *pipe;
     unsigned int         options;    /* pipe options */
+    struct async_queue   listen_q;   /* listen queue */
 };
 
 struct pipe_client
 {
     struct pipe_end      pipe_end;   /* common header for pipe_client and pipe_server */
-    struct pipe_server  *server;     /* server that this client is connected to */
     unsigned int         flags;      /* file flags */
 };
 
@@ -143,22 +133,23 @@ static const struct object_ops named_pipe_ops =
 };
 
 /* common server and client pipe end functions */
+static void pipe_end_destroy( struct object *obj );
 static enum server_fd_type pipe_end_get_fd_type( struct fd *fd );
 static struct fd *pipe_end_get_fd( struct object *obj );
+static struct security_descriptor *pipe_end_get_sd( struct object *obj );
+static int pipe_end_set_sd( struct object *obj, const struct security_descriptor *sd,
+                            unsigned int set_info );
 static int pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos );
 static int pipe_end_write( struct fd *fd, struct async *async_data, file_pos_t pos );
 static int pipe_end_flush( struct fd *fd, struct async *async );
 static void pipe_end_get_volume_info( struct fd *fd, unsigned int info_class );
 static void pipe_end_reselect_async( struct fd *fd, struct async_queue *queue );
+static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class );
 
 /* server end functions */
 static void pipe_server_dump( struct object *obj, int verbose );
-static struct security_descriptor *pipe_server_get_sd( struct object *obj );
-static int pipe_server_set_sd( struct object *obj, const struct security_descriptor *sd,
-                               unsigned int set_info );
 static void pipe_server_destroy( struct object *obj);
 static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
-static void pipe_server_get_file_info( struct fd *fd, unsigned int info_class );
 
 static const struct object_ops pipe_server_ops =
 {
@@ -172,8 +163,8 @@ static const struct object_ops pipe_server_ops =
     no_signal,                    /* signal */
     pipe_end_get_fd,              /* get_fd */
     default_fd_map_access,        /* map_access */
-    pipe_server_get_sd,           /* get_sd */
-    pipe_server_set_sd,           /* set_sd */
+    pipe_end_get_sd,              /* get_sd */
+    pipe_end_set_sd,              /* set_sd */
     no_lookup_name,               /* lookup_name */
     no_link_name,                 /* link_name */
     NULL,                         /* unlink_name */
@@ -190,7 +181,7 @@ static const struct fd_ops pipe_server_fd_ops =
     pipe_end_read,                /* read */
     pipe_end_write,               /* write */
     pipe_end_flush,               /* flush */
-    pipe_server_get_file_info,    /* get_file_info */
+    pipe_end_get_file_info,       /* get_file_info */
     pipe_end_get_volume_info,     /* get_volume_info */
     pipe_server_ioctl,            /* ioctl */
     no_fd_queue_async,            /* queue_async */
@@ -199,12 +190,7 @@ static const struct fd_ops pipe_server_fd_ops =
 
 /* client end functions */
 static void pipe_client_dump( struct object *obj, int verbose );
-static struct security_descriptor *pipe_client_get_sd( struct object *obj );
-static int pipe_client_set_sd( struct object *obj, const struct security_descriptor *sd,
-                               unsigned int set_info );
-static void pipe_client_destroy( struct object *obj );
 static int pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
-static void pipe_client_get_file_info( struct fd *fd, unsigned int info_class );
 
 static const struct object_ops pipe_client_ops =
 {
@@ -218,14 +204,14 @@ static const struct object_ops pipe_client_ops =
     no_signal,                    /* signal */
     pipe_end_get_fd,              /* get_fd */
     default_fd_map_access,        /* map_access */
-    pipe_client_get_sd,           /* get_sd */
-    pipe_client_set_sd,           /* set_sd */
+    pipe_end_get_sd,              /* get_sd */
+    pipe_end_set_sd,              /* set_sd */
     no_lookup_name,               /* lookup_name */
     no_link_name,                 /* link_name */
     NULL,                         /* unlink_name */
     no_open_file,                 /* open_file */
     fd_close_handle,              /* close_handle */
-    pipe_client_destroy           /* destroy */
+    pipe_end_destroy              /* destroy */
 };
 
 static const struct fd_ops pipe_client_fd_ops =
@@ -236,7 +222,7 @@ static const struct fd_ops pipe_client_fd_ops =
     pipe_end_read,                /* read */
     pipe_end_write,               /* write */
     pipe_end_flush,               /* flush */
-    pipe_client_get_file_info,    /* get_file_info */
+    pipe_end_get_file_info,       /* get_file_info */
     pipe_end_get_volume_info,     /* get_volume_info */
     pipe_client_ioctl,            /* ioctl */
     no_fd_queue_async,            /* queue_async */
@@ -309,14 +295,15 @@ static void pipe_server_dump( struct object *obj, int verbose )
 {
     struct pipe_server *server = (struct pipe_server *) obj;
     assert( obj->ops == &pipe_server_ops );
-    fprintf( stderr, "Named pipe server pipe=%p state=%d\n", server->pipe, server->state );
+    fprintf( stderr, "Named pipe server pipe=%p state=%d\n", server->pipe_end.pipe,
+             server->pipe_end.state );
 }
 
 static void pipe_client_dump( struct object *obj, int verbose )
 {
     struct pipe_client *client = (struct pipe_client *) obj;
     assert( obj->ops == &pipe_client_ops );
-    fprintf( stderr, "Named pipe client server=%p\n", client->server );
+    fprintf( stderr, "Named pipe client server=%p\n", client->pipe_end.connection );
 }
 
 static void named_pipe_destroy( struct object *obj)
@@ -334,26 +321,6 @@ static struct fd *pipe_end_get_fd( struct object *obj )
     return (struct fd *) grab_object( pipe_end->fd );
 }
 
-static void set_server_state( struct pipe_server *server, enum pipe_state state )
-{
-    server->state = state;
-
-    switch(state)
-    {
-    case ps_connected_server:
-    case ps_wait_disconnect:
-        break;
-    case ps_wait_open:
-    case ps_idle_server:
-        set_no_fd_status( server->pipe_end.fd, STATUS_PIPE_LISTENING );
-        break;
-    case ps_wait_connect:
-        set_no_fd_status( server->pipe_end.fd, STATUS_PIPE_DISCONNECTED );
-        break;
-    }
-}
-
-
 static struct pipe_message *queue_message( struct pipe_end *pipe_end, struct iosb *iosb )
 {
     struct pipe_message *message;
@@ -394,6 +361,8 @@ static void pipe_end_disconnect( struct pipe_end *pipe_end, unsigned int status
 
     pipe_end->connection = NULL;
 
+    pipe_end->state = status == STATUS_PIPE_DISCONNECTED
+        ? FILE_PIPE_DISCONNECTED_STATE : FILE_PIPE_CLOSING_STATE;
     fd_async_wake_up( pipe_end->fd, ASYNC_TYPE_WAIT, status );
     async_wake_up( &pipe_end->read_q, status );
     LIST_FOR_EACH_ENTRY_SAFE( message, next, &pipe_end->message_queue, struct pipe_message, entry )
@@ -413,10 +382,13 @@ static void pipe_end_disconnect( struct pipe_end *pipe_end, unsigned int status
     }
 }
 
-static void pipe_end_destroy( struct pipe_end *pipe_end )
+static void pipe_end_destroy( struct object *obj )
 {
+    struct pipe_end *pipe_end = (struct pipe_end *)obj;
     struct pipe_message *message;
 
+    pipe_end_disconnect( pipe_end, STATUS_PIPE_BROKEN );
+
     while (!list_empty( &pipe_end->message_queue ))
     {
         message = LIST_ENTRY( list_head(&pipe_end->message_queue), struct pipe_message, entry );
@@ -427,60 +399,22 @@ static void pipe_end_destroy( struct pipe_end *pipe_end )
     free_async_queue( &pipe_end->read_q );
     free_async_queue( &pipe_end->write_q );
     if (pipe_end->fd) release_object( pipe_end->fd );
+    if (pipe_end->pipe) release_object( pipe_end->pipe );
 }
 
-static void pipe_server_destroy( struct object *obj)
+static void pipe_server_destroy( struct object *obj )
 {
     struct pipe_server *server = (struct pipe_server *)obj;
+    struct named_pipe *pipe = server->pipe_end.pipe;
 
     assert( obj->ops == &pipe_server_ops );
 
-    pipe_end_disconnect( &server->pipe_end, STATUS_PIPE_BROKEN );
-
-    pipe_end_destroy( &server->pipe_end );
-    if (server->client)
-    {
-        server->client->server = NULL;
-        server->client = NULL;
-    }
-
-    assert( server->pipe->instances );
-    server->pipe->instances--;
-
+    assert( pipe->instances );
+    if (!--pipe->instances) unlink_named_object( &pipe->obj );
     list_remove( &server->entry );
-    release_object( server->pipe );
-}
 
-static void pipe_client_destroy( struct object *obj)
-{
-    struct pipe_client *client = (struct pipe_client *)obj;
-    struct pipe_server *server = client->server;
-
-    assert( obj->ops == &pipe_client_ops );
-
-    pipe_end_disconnect( &client->pipe_end, STATUS_PIPE_BROKEN );
-
-    if (server)
-    {
-        switch(server->state)
-        {
-        case ps_connected_server:
-            /* Don't destroy the server's fd here as we can't
-               do a successful flush without it. */
-            set_server_state( server, ps_wait_disconnect );
-            break;
-        case ps_idle_server:
-        case ps_wait_open:
-        case ps_wait_disconnect:
-        case ps_wait_connect:
-            assert( 0 );
-        }
-        assert( server->client );
-        server->client = NULL;
-        client->server = NULL;
-    }
-
-    pipe_end_destroy( &client->pipe_end );
+    free_async_queue( &server->listen_q );
+    pipe_end_destroy( obj );
 }
 
 static void named_pipe_device_dump( struct object *obj, int verbose )
@@ -567,8 +501,17 @@ static int pipe_end_flush( struct fd *fd, struct async *async )
     return 1;
 }
 
-static void pipe_end_get_file_info( struct fd *fd, struct named_pipe *pipe, unsigned int info_class )
+static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class )
 {
+    struct pipe_end *pipe_end = get_fd_user( fd );
+    struct named_pipe *pipe = pipe_end->pipe;
+
+    if (!pipe)
+    {
+        set_error( STATUS_PIPE_DISCONNECTED );
+        return;
+    }
+
     switch (info_class)
     {
     case FileNameInformation:
@@ -584,6 +527,12 @@ static void pipe_end_get_file_info( struct fd *fd, struct named_pipe *pipe, unsi
             }
 
             name = get_object_name( &pipe->obj, &name_len );
+            /* FIXME: We should be able to return on unlinked pipe */
+            if (!name)
+            {
+                set_error( STATUS_PIPE_DISCONNECTED );
+                return;
+            }
             reply_size = offsetof( FILE_NAME_INFORMATION, FileName[name_len/sizeof(WCHAR) + 1] );
             if (reply_size > get_reply_max_size())
             {
@@ -603,49 +552,23 @@ static void pipe_end_get_file_info( struct fd *fd, struct named_pipe *pipe, unsi
     }
 }
 
-static struct security_descriptor *pipe_server_get_sd( struct object *obj )
+static struct security_descriptor *pipe_end_get_sd( struct object *obj )
 {
-    struct pipe_server *server = (struct pipe_server *) obj;
-    return default_get_sd( &server->pipe->obj );
-}
-
-static struct security_descriptor *pipe_client_get_sd( struct object *obj )
-{
-    struct pipe_client *client = (struct pipe_client *) obj;
-    if (client->server) return default_get_sd( &client->server->pipe->obj );
+    struct pipe_end *pipe_end = (struct pipe_end *) obj;
+    if (pipe_end->pipe) return default_get_sd( &pipe_end->pipe->obj );
     set_error( STATUS_PIPE_DISCONNECTED );
     return NULL;
 }
 
-static int pipe_server_set_sd( struct object *obj, const struct security_descriptor *sd,
-                               unsigned int set_info )
+static int pipe_end_set_sd( struct object *obj, const struct security_descriptor *sd,
+                            unsigned int set_info )
 {
-    struct pipe_server *server = (struct pipe_server *) obj;
-    return default_set_sd( &server->pipe->obj, sd, set_info );
-}
-
-static int pipe_client_set_sd( struct object *obj, const struct security_descriptor *sd,
-                               unsigned int set_info )
-{
-    struct pipe_client *client = (struct pipe_client *) obj;
-    if (client->server) return default_set_sd( &client->server->pipe->obj, sd, set_info );
+    struct pipe_end *pipe_end = (struct pipe_end *) obj;
+    if (pipe_end->pipe) return default_set_sd( &pipe_end->pipe->obj, sd, set_info );
     set_error( STATUS_PIPE_DISCONNECTED );
     return 0;
 }
 
-static void pipe_server_get_file_info( struct fd *fd, unsigned int info_class )
-{
-    struct pipe_server *server = get_fd_user( fd );
-    pipe_end_get_file_info( fd, server->pipe, info_class );
-}
-
-static void pipe_client_get_file_info( struct fd *fd, unsigned int info_class )
-{
-    struct pipe_client *client = get_fd_user( fd );
-    if (client->server) pipe_end_get_file_info( fd, client->server->pipe, info_class );
-    else set_error( STATUS_PIPE_DISCONNECTED );
-}
-
 static void pipe_end_get_volume_info( struct fd *fd, unsigned int info_class )
 {
     switch (info_class)
@@ -795,8 +718,18 @@ static int pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
 
-    if (!pipe_end->connection && list_empty( &pipe_end->message_queue ))
+    switch (pipe_end->state)
     {
+    case FILE_PIPE_CONNECTED_STATE:
+        break;
+    case FILE_PIPE_DISCONNECTED_STATE:
+        set_error( STATUS_PIPE_DISCONNECTED );
+        return 0;
+    case FILE_PIPE_LISTENING_STATE:
+        set_error( STATUS_PIPE_LISTENING );
+        return 0;
+    case FILE_PIPE_CLOSING_STATE:
+        if (!list_empty( &pipe_end->message_queue )) break;
         set_error( STATUS_PIPE_BROKEN );
         return 0;
     }
@@ -813,10 +746,19 @@ static int pipe_end_write( struct fd *fd, struct async *async, file_pos_t pos )
     struct pipe_message *message;
     struct iosb *iosb;
 
-    if (!pipe_end->connection)
+    switch (pipe_end->state)
     {
+    case FILE_PIPE_CONNECTED_STATE:
+        break;
+    case FILE_PIPE_DISCONNECTED_STATE:
         set_error( STATUS_PIPE_DISCONNECTED );
         return 0;
+    case FILE_PIPE_LISTENING_STATE:
+        set_error( STATUS_PIPE_LISTENING );
+        return 0;
+    case FILE_PIPE_CLOSING_STATE:
+        set_error( STATUS_PIPE_CLOSING );
+        return 0;
     }
 
     if (!(pipe_end->flags & NAMED_PIPE_MESSAGE_STREAM_WRITE) && !get_req_data_size()) return 1;
@@ -865,10 +807,17 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
     }
     reply_size -= offsetof( FILE_PIPE_PEEK_BUFFER, Data );
 
-    if (!pipe_end->connection && list_empty( &pipe_end->message_queue ))
+    switch (pipe_end->state)
     {
+    case FILE_PIPE_CONNECTED_STATE:
+        break;
+    case FILE_PIPE_CLOSING_STATE:
+        if (!list_empty( &pipe_end->message_queue )) break;
         set_error( STATUS_PIPE_BROKEN );
         return 0;
+    default:
+        set_error( STATUS_INVALID_PIPE_STATE );
+        return 0;
     }
 
     LIST_FOR_EACH_ENTRY( message, &pipe_end->message_queue, struct pipe_message, entry )
@@ -883,7 +832,7 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
     }
 
     if (!(buffer = set_reply_data_size( offsetof( FILE_PIPE_PEEK_BUFFER, Data[reply_size] )))) return 0;
-    buffer->NamedPipeState    = 0;  /* FIXME */
+    buffer->NamedPipeState    = pipe_end->state;
     buffer->ReadDataAvailable = avail;
     buffer->NumberOfMessages  = 0;  /* FIXME */
     buffer->MessageLength     = message_length;
@@ -908,6 +857,12 @@ static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
     struct pipe_message *message;
     struct iosb *iosb;
 
+    if (!pipe_end->connection)
+    {
+        set_error( STATUS_INVALID_PIPE_STATE );
+        return 0;
+    }
+
     if ((pipe_end->flags & (NAMED_PIPE_MESSAGE_STREAM_WRITE | NAMED_PIPE_MESSAGE_STREAM_READ))
         != (NAMED_PIPE_MESSAGE_STREAM_WRITE | NAMED_PIPE_MESSAGE_STREAM_READ))
     {
@@ -915,12 +870,6 @@ static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
         return 0;
     }
 
-    if (!pipe_end->connection)
-    {
-        set_error( STATUS_PIPE_BROKEN );
-        return 0;
-    }
-
     /* not allowed if we already have read data buffered */
     if (!list_empty( &pipe_end->message_queue ))
     {
@@ -1000,52 +949,45 @@ static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
     switch(code)
     {
     case FSCTL_PIPE_LISTEN:
-        switch(server->state)
+        switch(server->pipe_end.state)
         {
-        case ps_idle_server:
-        case ps_wait_connect:
-            fd_queue_async( server->pipe_end.fd, async, ASYNC_TYPE_WAIT );
-            set_server_state( server, ps_wait_open );
-            async_wake_up( &server->pipe->waiters, STATUS_SUCCESS );
-            set_error( STATUS_PENDING );
-            return 1;
-        case ps_connected_server:
+        case FILE_PIPE_LISTENING_STATE:
+        case FILE_PIPE_DISCONNECTED_STATE:
+            break;
+        case FILE_PIPE_CONNECTED_STATE:
             set_error( STATUS_PIPE_CONNECTED );
-            break;
-        case ps_wait_disconnect:
-            set_error( STATUS_NO_DATA_DETECTED );
-            break;
-        case ps_wait_open:
-            set_error( STATUS_INVALID_HANDLE );
-            break;
+            return 0;
+        case FILE_PIPE_CLOSING_STATE:
+            set_error( STATUS_PIPE_CLOSING );
+            return 0;
         }
-        return 0;
+
+        server->pipe_end.state = FILE_PIPE_LISTENING_STATE;
+        queue_async( &server->listen_q, async );
+        async_wake_up( &server->pipe_end.pipe->waiters, STATUS_SUCCESS );
+        set_error( STATUS_PENDING );
+        return 1;
 
     case FSCTL_PIPE_DISCONNECT:
-        switch(server->state)
+        switch(server->pipe_end.state)
         {
-        case ps_connected_server:
-            assert( server->client );
-
-            /* dump the client and server fds - client loses all waiting data */
-            pipe_end_disconnect( &server->pipe_end, STATUS_PIPE_DISCONNECTED );
-            server->client->server = NULL;
-            server->client = NULL;
-            set_server_state( server, ps_wait_connect );
+        case FILE_PIPE_CONNECTED_STATE:
+            /* dump the client connection - all data is lost */
+            assert( server->pipe_end.connection );
+            release_object( server->pipe_end.connection->pipe );
+            server->pipe_end.connection->pipe = NULL;
             break;
-        case ps_wait_disconnect:
-            assert( !server->client );
-            pipe_end_disconnect( &server->pipe_end, STATUS_PIPE_DISCONNECTED );
-            set_server_state( server, ps_wait_connect );
+        case FILE_PIPE_CLOSING_STATE:
             break;
-        case ps_idle_server:
-        case ps_wait_open:
+        case FILE_PIPE_LISTENING_STATE:
             set_error( STATUS_PIPE_LISTENING );
             return 0;
-        case ps_wait_connect:
+        case FILE_PIPE_DISCONNECTED_STATE:
             set_error( STATUS_PIPE_DISCONNECTED );
             return 0;
         }
+
+        pipe_end_disconnect( &server->pipe_end, STATUS_PIPE_DISCONNECTED );
         return 1;
 
     default:
@@ -1068,16 +1010,10 @@ static int pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
     }
 }
 
-static struct pipe_server *get_pipe_server_obj( struct process *process,
-                                obj_handle_t handle, unsigned int access )
-{
-    struct object *obj;
-    obj = get_handle_obj( process, handle, access, &pipe_server_ops );
-    return (struct pipe_server *) obj;
-}
-
-static void init_pipe_end( struct pipe_end *pipe_end, unsigned int pipe_flags, data_size_t buffer_size )
+static void init_pipe_end( struct pipe_end *pipe_end, struct named_pipe *pipe,
+                           unsigned int pipe_flags, data_size_t buffer_size )
 {
+    pipe_end->pipe = (struct named_pipe *)grab_object( pipe );
     pipe_end->fd = NULL;
     pipe_end->flags = pipe_flags;
     pipe_end->connection = NULL;
@@ -1096,25 +1032,24 @@ static struct pipe_server *create_pipe_server( struct named_pipe *pipe, unsigned
     if (!server)
         return NULL;
 
-    server->pipe = pipe;
-    server->client = NULL;
     server->options = options;
-    init_pipe_end( &server->pipe_end, pipe_flags, pipe->insize );
+    init_pipe_end( &server->pipe_end, pipe, pipe_flags, pipe->insize );
+    server->pipe_end.state = FILE_PIPE_LISTENING_STATE;
     server->pipe_end.server_pid = get_process_id( current->process );
 
     list_add_head( &pipe->servers, &server->entry );
-    grab_object( pipe );
     if (!(server->pipe_end.fd = alloc_pseudo_fd( &pipe_server_fd_ops, &server->pipe_end.obj, options )))
     {
         release_object( server );
         return NULL;
     }
+    allow_fd_caching( server->pipe_end.fd );
     set_fd_signaled( server->pipe_end.fd, 1 );
-    set_server_state( server, ps_idle_server );
+    init_async_queue( &server->listen_q );
     return server;
 }
 
-static struct pipe_client *create_pipe_client( unsigned int flags, unsigned int pipe_flags,
+static struct pipe_client *create_pipe_client( unsigned int flags, struct named_pipe *pipe,
                                                data_size_t buffer_size, unsigned int options )
 {
     struct pipe_client *client;
@@ -1123,9 +1058,9 @@ static struct pipe_client *create_pipe_client( unsigned int flags, unsigned int
     if (!client)
         return NULL;
 
-    client->server = NULL;
     client->flags = flags;
-    init_pipe_end( &client->pipe_end, pipe_flags, buffer_size );
+    init_pipe_end( &client->pipe_end, pipe, pipe->flags, buffer_size );
+    client->pipe_end.state = FILE_PIPE_CONNECTED_STATE;
     client->pipe_end.client_pid = get_process_id( current->process );
 
     client->pipe_end.fd = alloc_pseudo_fd( &pipe_client_fd_ops, &client->pipe_end.obj, options );
@@ -1147,14 +1082,14 @@ static struct pipe_server *find_available_server( struct named_pipe *pipe )
     /* look for pipe servers that are listening */
     LIST_FOR_EACH_ENTRY( server, &pipe->servers, struct pipe_server, entry )
     {
-        if (server->state == ps_wait_open)
+        if (server->pipe_end.state == FILE_PIPE_LISTENING_STATE && async_queued( &server->listen_q ))
             return (struct pipe_server *)grab_object( server );
     }
 
     /* fall back to pipe servers that are idle */
     LIST_FOR_EACH_ENTRY( server, &pipe->servers, struct pipe_server, entry )
     {
-        if (server->state == ps_idle_server)
+        if (server->pipe_end.state == FILE_PIPE_LISTENING_STATE )
             return (struct pipe_server *)grab_object( server );
     }
 
@@ -1189,7 +1124,7 @@ static struct object *named_pipe_open_file( struct object *obj, unsigned int acc
         return NULL;
     }
 
-    pipe_sharing = server->pipe->sharing;
+    pipe_sharing = pipe->sharing;
     if (((access & GENERIC_READ) && !(pipe_sharing & FILE_SHARE_READ)) ||
         ((access & GENERIC_WRITE) && !(pipe_sharing & FILE_SHARE_WRITE)))
     {
@@ -1198,15 +1133,10 @@ static struct object *named_pipe_open_file( struct object *obj, unsigned int acc
         return NULL;
     }
 
-    if ((client = create_pipe_client( options, pipe->flags, pipe->outsize, options )))
+    if ((client = create_pipe_client( options, pipe, pipe->outsize, options )))
     {
-        set_no_fd_status( server->pipe_end.fd, STATUS_BAD_DEVICE_TYPE );
-        allow_fd_caching( server->pipe_end.fd );
-        if (server->state == ps_wait_open)
-            fd_async_wake_up( server->pipe_end.fd, ASYNC_TYPE_WAIT, STATUS_SUCCESS );
-        set_server_state( server, ps_connected_server );
-        server->client = client;
-        client->server = server;
+        async_wake_up( &server->listen_q, STATUS_SUCCESS );
+        server->pipe_end.state = FILE_PIPE_CONNECTED_STATE;
         server->pipe_end.connection = &client->pipe_end;
         client->pipe_end.connection = &server->pipe_end;
         server->pipe_end.client_pid = client->pipe_end.client_pid;
@@ -1334,7 +1264,7 @@ DECL_HANDLER(create_named_pipe)
     if (server)
     {
         reply->handle = alloc_handle( current->process, server, req->access, objattr->attributes );
-        server->pipe->instances++;
+        pipe->instances++;
         release_object( server );
     }
 
@@ -1343,79 +1273,63 @@ DECL_HANDLER(create_named_pipe)
 
 DECL_HANDLER(get_named_pipe_info)
 {
-    struct pipe_server *server;
-    struct pipe_client *client = NULL;
+    struct pipe_end *pipe_end;
 
-    server = get_pipe_server_obj( current->process, req->handle, FILE_READ_ATTRIBUTES );
-    if (!server)
+    pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
+                                                  FILE_READ_ATTRIBUTES, &pipe_server_ops );
+    if (!pipe_end)
     {
         if (get_error() != STATUS_OBJECT_TYPE_MISMATCH)
             return;
 
         clear_error();
-        client = (struct pipe_client *)get_handle_obj( current->process, req->handle,
-                                                       0, &pipe_client_ops );
-        if (!client) return;
-        server = client->server;
+        pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
+                                                      FILE_READ_ATTRIBUTES, &pipe_client_ops );
+        if (!pipe_end) return;
     }
 
-    reply->flags = client ? client->pipe_end.flags : server->pipe_end.flags;
-    if (server)
+    if (pipe_end->pipe)
     {
-        reply->sharing      = server->pipe->sharing;
-        reply->maxinstances = server->pipe->maxinstances;
-        reply->instances    = server->pipe->instances;
-        reply->insize       = server->pipe->insize;
-        reply->outsize      = server->pipe->outsize;
-    }
+        reply->flags        = pipe_end->flags;
+        reply->sharing      = pipe_end->pipe->sharing;
+        reply->maxinstances = pipe_end->pipe->maxinstances;
+        reply->instances    = pipe_end->pipe->instances;
+        reply->insize       = pipe_end->pipe->insize;
+        reply->outsize      = pipe_end->pipe->outsize;
 
-    if (client)
-        release_object(client);
-    else
-    {
-        reply->flags |= NAMED_PIPE_SERVER_END;
-        release_object(server);
+        if (pipe_end->obj.ops == &pipe_server_ops) reply->flags |= NAMED_PIPE_SERVER_END;
     }
+    else set_error( STATUS_PIPE_DISCONNECTED );
+
+    release_object( pipe_end );
 }
 
 DECL_HANDLER(set_named_pipe_info)
 {
-    struct pipe_server *server;
-    struct pipe_client *client = NULL;
+    struct pipe_end *pipe_end;
 
-    server = get_pipe_server_obj( current->process, req->handle, FILE_WRITE_ATTRIBUTES );
-    if (!server)
+    pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
+                                                  FILE_WRITE_ATTRIBUTES, &pipe_server_ops );
+    if (!pipe_end)
     {
         if (get_error() != STATUS_OBJECT_TYPE_MISMATCH)
             return;
 
         clear_error();
-        client = (struct pipe_client *)get_handle_obj( current->process, req->handle,
-                                                       0, &pipe_client_ops );
-        if (!client) return;
-        if (!(server = client->server))
-        {
-            release_object( client );
-            return;
-        }
+        pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
+                                                      0, &pipe_client_ops );
+        if (!pipe_end) return;
     }
 
     if ((req->flags & ~(NAMED_PIPE_MESSAGE_STREAM_READ | NAMED_PIPE_NONBLOCKING_MODE)) ||
-            ((req->flags & NAMED_PIPE_MESSAGE_STREAM_READ) && !(server->pipe->flags & NAMED_PIPE_MESSAGE_STREAM_WRITE)))
+            ((req->flags & NAMED_PIPE_MESSAGE_STREAM_READ) && !(pipe_end->pipe->flags & NAMED_PIPE_MESSAGE_STREAM_WRITE)))
     {
         set_error( STATUS_INVALID_PARAMETER );
     }
-    else if (client)
-    {
-        client->pipe_end.flags = server->pipe->flags | req->flags;
-    }
     else
     {
-        server->pipe_end.flags = server->pipe->flags | req->flags;
+        pipe_end->flags = pipe_end->pipe->flags | req->flags;
     }
 
-    if (client)
-        release_object(client);
-    else
-        release_object(server);
+    release_object( pipe_end );
 }
